// Chrono68k v0.51
// CopyLeft by Jonas Gehring & Ilidio Martins

// See the readme.txt file for details!

// C Source File
// Created 29.09.04; 21:04:11


#include <graph.h>						// Common (non-windowed) graphic routines
#include <stdio.h>						// ANSI-compatible file and TTY input/output routines
#include <system.h>						// System routines (for accessing system timers, queues, etc.)
#include <compat.h>						// Various calc-dependent and AMS-dependent pseudo-constants
#include <kbd.h>							// Keyboard handling routines
#include <statline.h>					// Routines for manipulation of the status line
#include <asmtypes.h>					// Common type definitions known to ASM programmers
#include <intr.h>							// Routines for creating interrupt handlers
#include <gray.h>							// Routines for grayscale graphics

#include "unknown.h"					// I modified this file to accept the new TIClock options

#include "extgraph.h"					// Include ExtGraph Library v2.00beta3 by TI-Chess Team

#include "chrono68k.h"


// Constant for the Font: ("small" for TI89, "medium" for the other 68ks)
#ifndef CALCULATOR
#define FONT F_4x6
#else
#define FONT F_6x8
#endif

BOOL *timemode = ANALOG;


// Clears the keyboard queue
void ClearQueue(void)
{
	while (_rowread(0)) continue;
}


// Get the current time
void TimeInput(void) 
{
	unsigned short year, month, day, ho, mi, se;

	// Turns on the TIClock
	ClockOn();
	
	DateAndTime_Get(&year, &month, &day, &ho, &mi, &se);
	
	// Apply the time to the  globals
	hour = ho; minu = mi; seco = se;
	year_=year; 
	month_=month;
	day_=day;
}


// The Counter
CALLBACK void CountTime(void) {
	// Fill temporary variables
	secot = seco;
	minut = minu;
	hourt = hour;
		
	// Add another second
	seco++;
	
	if (seco > 58) {
		seco = 0;
		minu++;
		if (minu > 58) {
				minu = 0;
				hour++;
				if (hour > 23)
					hour = 0;
		}
	}
	
	// Display the changed time in the specific format
	if (*timemode == ANALOG)
		DisplayTimeAnalog();
	else
		DisplayTimeDigital(FALSE);
}


// The Counter for the stop watch
CALLBACK void StopTime(void) {
	char str[10];
	
	// Add another 1/10 second
	stopss++;
	
	if (stopss > 9) {
		stopss = 0;
		stops++;
		
		if (stops > 59) {
			stops = 0;
			stopm++;
			
			if (stopm > 59)
				stopm = 0;
		}
	}
	
	
	// Display the time which is stopped in letters
	sprintf(str, "%d%d:%d%d:%d", stopm/10, stopm-((stopm/10)*10), stops/10, stops-((stops/10)*10), stopss);

	if (*timemode == ANALOG)
		GrayDrawStrExt2B(0, LCD_HEIGHT/2+40, str, A_REPLACE, FONT, GrayGetPlane(LIGHT_PLANE), GrayGetPlane(DARK_PLANE));
	else
		GrayDrawStrExt2B(0, LCD_HEIGHT/2+25, str, A_REPLACE | A_CENTERED, FONT, GrayGetPlane(LIGHT_PLANE), GrayGetPlane(DARK_PLANE));
}


// Sets the format of the Time (analog or digital)
void SetTimeFormat(void)
{
	BOOL opt, optt;
	
	opt = *timemode;
	
	DrawMenu(FALSE);
	GrayDrawStrExt((LCD_WIDTH/2-27), (LCD_HEIGHT/2)-8, "Analog Clock", A_XOR, F_4x6);
	GrayDrawStrExt((LCD_WIDTH/2-27), (LCD_HEIGHT/2)+10,  "Digital Clock", A_XOR, F_4x6);
	
	GrayDrawStrExt((LCD_WIDTH/2-40), (LCD_HEIGHT/2)-8+(opt*18), "-", A_XOR, F_4x6);
	
	do {
		optt = opt;
		
		// The user can change the time mode
		if (_keytest(RR_UP) || _keytest(RR_DOWN)) {
			ClearQueue();
			if (opt == ANALOG)
				opt = DIGITAL;
			else
				opt = ANALOG;
		}
	
		// The time mode has been changed
		if (_keytest(RR_ENTER) || _keytest(RR_ENTER2)) {
			ClearQueue();
			*timemode = opt;
			return;
		}
		
		if (_keytest(RR_ESC)) {
			ClearQueue();
			return;
		}
	
		// Redraw cursor
		if (opt != optt) {
			GrayDrawStrExt((LCD_WIDTH/2-40), (LCD_HEIGHT/2)-8+(optt*18), "-", A_XOR, F_4x6);
			GrayDrawStrExt((LCD_WIDTH/2-40), (LCD_HEIGHT/2)-8+(opt*18), "-", A_XOR, F_4x6);
		}
	
	}	while (TRUE);
}


// Restores the screen with a clock running
void InitScreen(void)
{
	char str[11];
	
	if (*timemode == DIGITAL) {
		ClearGrayScreen();
		DisplayTimeDigital(TRUE);
	} else {
		DrawClock();
		DisplayTimeAnalog();
	}	

	// Display the time which is stopped in letters
	sprintf(str, "%d%d:%d%d:%d", stopm/10, stopm-((stopm/10)*10), stops/10, stops-((stops/10)*10), stopss);

	if (*timemode == ANALOG)
		GrayDrawStrExt2B(0, LCD_HEIGHT/2+40, str, A_REPLACE, FONT, GrayGetPlane(LIGHT_PLANE), GrayGetPlane(DARK_PLANE));
	else
		GrayDrawStrExt2B(0, LCD_HEIGHT/2+25, str, A_REPLACE | A_CENTERED, FONT, GrayGetPlane(LIGHT_PLANE), GrayGetPlane(DARK_PLANE));
}


// Main Function
void _main(void)
{
	LCD_BUFFER scrsave;
	BOOL StopOn = FALSE;
	char stmsg[33];
	INT_HANDLER save_int_5 = GetIntVec(AUTO_INT_5);
	
	LCD_save(scrsave);

	save_int_1 = GetIntVec(AUTO_INT_1);

	// Disabling of Auto-Int 1 and 5 to aviod displaying of the alpha-, diamond-,
	// second-, shift-symbols in the status line
	SetIntVec(AUTO_INT_1, DUMMY_HANDLER);   // redirect auto-int 1 to "nothing"
	SetIntVec(AUTO_INT_5, DUMMY_HANDLER);   // redirect auto-int 5 to "nothing"
	
	if (!GrayOn())
		return;

	// Get the current time
	TimeInput();

	// Load settings
	timemode = LoadFromFile();

	InitScreen();

 	// Register the timer for the Counter 
	OSVRegisterTimer(1, 20, CountTime);
	
	do {
		// Turn the stop watch OFF/ON
		if (_keytest(RR_HAND) || _keytest(RR_ALPHA)) {
			ClearQueue();
			if (StopOn) {
				StopOn = FALSE;
				OSVFreeTimer(2);
			} else {
				StopOn = TRUE;
				OSVRegisterTimer(2, 2, StopTime);
			}
		}
		
		// Reset the stop watch
		if (_keytest(RR_CLEAR)) {
			ClearQueue();
			stopss = 0; stops = 0; stopm = 0;
			if(!StopOn) {
				if (*timemode == ANALOG)
					GrayDrawStrExt2B(0, LCD_HEIGHT/2+40, "00:00:0", A_REPLACE, FONT, GrayGetPlane(LIGHT_PLANE), GrayGetPlane(DARK_PLANE));
				else
					GrayDrawStrExt2B(0, LCD_HEIGHT/2+25, "00:00:0", A_REPLACE | A_CENTERED, FONT, GrayGetPlane(LIGHT_PLANE), GrayGetPlane(DARK_PLANE));
			}
		}
		
		// Open the menu
		if (_keytest(RR_ENTER) || _keytest(RR_ENTER2)) {
			ClearQueue();
			// Free the timers
			OSVFreeTimer(1);
			OSVFreeTimer(2);
			// Show the menu
			if (!Menu())
				break;
			// Get the current time
			TimeInput();
			InitScreen();		
			// Register the timers
			OSVRegisterTimer(1, 20, CountTime);
			if (StopOn)
				OSVRegisterTimer(2, 2, StopTime);
		}
			
			
		// Switch between analog and digital clock
		if (_keytest(RR_MODE)) {
			ClearQueue();
			
			if (*timemode == ANALOG)
				*timemode = DIGITAL;
			else
				*timemode = ANALOG;
			
			InitScreen();
		}
		
		
		// Adjust the contrast
		if (_keytest(RR_PLUS)) {
			ClearQueue();
			OSContrastUp();
		}
		if (_keytest(RR_MINUS)) {
			ClearQueue();
			OSContrastDn();
		}
		
		if (_keytest(RR_ESC)) {
			ClearQueue();
			break;
		}
					
	} while (TRUE);


	ClearQueue();
	// Free the timers
	OSVFreeTimer(1);
	OSVFreeTimer(2);
	
	// Save timemode
	SaveToFile(timemode);

	GrayOff();

	// Enable tha Auto-Ints again
	SetIntVec(AUTO_INT_1, save_int_1);
	SetIntVec(AUTO_INT_5, save_int_5);
	
	ClrScr();
	LCD_restore(scrsave);
	GKeyFlush();
	sprintf(stmsg, "Chrono68k %s by saubue & F34rFunny", VERSION);
	ST_helpMsg(stmsg);
}
